@@ -36,19 +36,22 @@
 #define D_BT_ABORT_OUTSTANDING_REQUEST_EVENT_H
 
 #include "common.h"
-#include "SharedHandle.h"
-#include "Piece.h"
+
+#include <memory>
 
 namespace aria2 {
 
+class Piece;
+
 class BtAbortOutstandingRequestEvent {
 private:
-  SharedHandle<Piece> piece_;
-public:
-  BtAbortOutstandingRequestEvent(const SharedHandle<Piece>& piece):
-    piece_(piece) {}
+  std::shared_ptr<Piece> piece_;
 
-  const SharedHandle<Piece>& getPiece() const { return piece_; }
+public:
+  BtAbortOutstandingRequestEvent(const std::shared_ptr<Piece>& piece);
+  ~BtAbortOutstandingRequestEvent();
+
+  const std::shared_ptr<Piece>& getPiece() const { return piece_; }
 };
 
 } // namespace aria2
