@@ -41,33 +41,63 @@
 #include "Command.h"
 #include "DownloadEngine.h"
 #include "DownloadContext.h"
-#include "RequestGroupMan.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
+#include "FileEntry.h"
+#include "PieceStorage.h"
+#include "DiskAdaptor.h"
+#include "Option.h"
+#include "prefs.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
-BtFileAllocationEntry::BtFileAllocationEntry(RequestGroup* requestGroup):
-  FileAllocationEntry(requestGroup, 0) {}
-
-BtFileAllocationEntry::~BtFileAllocationEntry() {}
-
-void BtFileAllocationEntry::prepareForNextAction
-(std::vector<Command*>& commands, DownloadEngine* e)
+BtFileAllocationEntry::BtFileAllocationEntry(RequestGroup* requestGroup)
+    : FileAllocationEntry(requestGroup, nullptr)
 {
-  BtSetup().setup(commands, getRequestGroup(), e,
-                  getRequestGroup()->getOption().get());
-  if(!getRequestGroup()->downloadFinished()) {
+}
+
+BtFileAllocationEntry::~BtFileAllocationEntry() = default;
+
+void BtFileAllocationEntry::prepareForNextAction(
+    std::vector<std::unique_ptr<Command>>& commands, DownloadEngine* e)
+{
+  auto& option = getRequestGroup()->getOption();
+
+  BtSetup().setup(commands, getRequestGroup(), e, option.get());
+  if (option->getAsBool(PREF_ENABLE_MMAP) &&
+      option->get(PREF_FILE_ALLOCATION) != V_NONE &&
+      getRequestGroup()->getPieceStorage()->getDiskAdaptor()->size() <=
+          option->getAsLLInt(PREF_MAX_MMAP_LIMIT)) {
+    getRequestGroup()->getPieceStorage()->getDiskAdaptor()->enableMmap();
+  }
+  if (!getRequestGroup()->downloadFinished()) {
     // For DownloadContext::resetDownloadStartTime(), see also
     // RequestGroup::createInitialCommand()
     getRequestGroup()->getDownloadContext()->resetDownloadStartTime();
-    const std::vector<SharedHandle<FileEntry> >& fileEntries =
-      getRequestGroup()->getDownloadContext()->getFileEntries();
-    if(isUriSuppliedForRequsetFileEntry
-       (fileEntries.begin(), fileEntries.end())) {
+    const std::vector<std::shared_ptr<FileEntry>>& fileEntries =
+        getRequestGroup()->getDownloadContext()->getFileEntries();
+    if (isUriSuppliedForRequsetFileEntry(std::begin(fileEntries),
+                                         std::end(fileEntries))) {
       getRequestGroup()->createNextCommandWithAdj(commands, e, 0);
     }
   }
+  else {
+#ifdef __MINGW32__
+    const std::shared_ptr<DiskAdaptor>& diskAdaptor =
+        getRequestGroup()->getPieceStorage()->getDiskAdaptor();
+    if (!diskAdaptor->isReadOnlyEnabled()) {
+      // On Windows, if aria2 opens files with GENERIC_WRITE access
+      // right, some programs cannot open them aria2 is seeding. To
+      // avoid this situation, re-open the files with read-only
+      // enabled.
+      A2_LOG_INFO("Closing files and re-open them with read-only mode"
+                  " enabled.");
+      diskAdaptor->closeFile();
+      diskAdaptor->enableReadOnly();
+      diskAdaptor->openFile();
+    }
+#endif // __MINGW32__
+    getRequestGroup()->enableSeedOnly();
+  }
 }
 
 } // namespace aria2
