@@ -32,22 +32,23 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DHT_NODE_LOOKUP_ENTRY_H_
-#define _D_DHT_NODE_LOOKUP_ENTRY_H_
+#ifndef D_DHT_NODE_LOOKUP_ENTRY_H
+#define D_DHT_NODE_LOOKUP_ENTRY_H
 
 #include "common.h"
-#include "SharedHandle.h"
+
+#include <memory>
 
 namespace aria2 {
 
 class DHTNode;
 
 struct DHTNodeLookupEntry {
-  SharedHandle<DHTNode> node;
+  std::shared_ptr<DHTNode> node;
 
   bool used;
 
-  DHTNodeLookupEntry(const SharedHandle<DHTNode>& node);
+  DHTNodeLookupEntry(const std::shared_ptr<DHTNode>& node);
 
   DHTNodeLookupEntry();
 
@@ -56,4 +57,4 @@ struct DHTNodeLookupEntry {
 
 } // namespace aria2;
 
-#endif // _D_DHT_NODE_LOOKUP_ENTRY_H_
+#endif // D_DHT_NODE_LOOKUP_ENTRY_H
