@@ -32,30 +32,36 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DL_ABORT_EX_H_
-#define _D_DL_ABORT_EX_H_
+#ifndef D_DL_ABORT_EX_H
+#define D_DL_ABORT_EX_H
 #include "RecoverableException.h"
 
 namespace aria2 {
 
-class DlAbortEx:public RecoverableException {
+class DlAbortEx : public RecoverableException {
 protected:
-  virtual SharedHandle<Exception> copy() const;
+  virtual std::shared_ptr<Exception> copy() const CXX11_OVERRIDE;
+
 public:
   DlAbortEx(const char* file, int line, const std::string& msg);
 
   DlAbortEx(const char* file, int line, const std::string& msg,
             const Exception& cause);
 
-  DlAbortEx(const char* file, int line, const RecoverableException& e);
-
   DlAbortEx(const char* file, int line, const std::string& msg,
-            downloadresultcode::RESULT code);
+            error_code::Value code);
+
+  DlAbortEx(const char* file, int line, int errNum, const std::string& msg);
+
+  DlAbortEx(const char* file, int line, int errNum, const std::string& msg,
+            error_code::Value code);
 };
 
 #define DL_ABORT_EX(arg) DlAbortEx(__FILE__, __LINE__, arg)
 #define DL_ABORT_EX2(arg1, arg2) DlAbortEx(__FILE__, __LINE__, arg1, arg2)
+#define DL_ABORT_EX3(arg1, arg2, arg3)                                         \
+  DlAbortEx(__FILE__, __LINE__, arg1, arg2, arg3)
 
 } // namespace aria2
 
-#endif // _D_DL_ABORT_EX_H_
+#endif // D_DL_ABORT_EX_H
