@@ -2,7 +2,7 @@
 /*
  * aria2 - The high speed download utility
  *
- * Copyright (C) 2006 Tatsuhiro Tsujikawa
+ * Copyright (C) 2012 Tatsuhiro Tsujikawa
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,58 +33,36 @@
  */
 /* copyright --> */
 #include "DownloadHandlerConstants.h"
-#include "array_fun.h"
 
 namespace aria2 {
 
-const char* DownloadHandlerConstants::METALINK_EXTENSIONS[] = {
-  ".metalink", // Metalink3Spec
-  ".meta4" // Metalink4Spec
-};
+namespace {
+const char* METALINK_EXTENSIONS[] = {".metalink", // Metalink3Spec
+                                     ".meta4",    // Metalink4Spec
+                                     nullptr};
+} // namespace
 
-const char* DownloadHandlerConstants::METALINK_CONTENT_TYPES[] = {
-  "application/metalink4+xml", // Metalink4Spec
-  "application/metalink+xml" // Metalink3Spec
-};
+const char** getMetalinkExtensions() { return METALINK_EXTENSIONS; }
 
-const char* DownloadHandlerConstants::BT_EXTENSIONS[] = { ".torrent" };
+namespace {
+const char* METALINK_CONTENT_TYPES[] = {
+    "application/metalink4+xml", // Metalink4Spec
+    "application/metalink+xml",  // Metalink3Spec
+    nullptr};
+} // namespace
 
-const char* DownloadHandlerConstants::BT_CONTENT_TYPES[] = {
-  "application/x-bittorrent"
-};
+const char** getMetalinkContentTypes() { return METALINK_CONTENT_TYPES; }
 
-const std::vector<std::string>&
-DownloadHandlerConstants::getMetalinkExtensions()
-{
-  static const std::vector<std::string> l
-    (&METALINK_EXTENSIONS[0],
-     &METALINK_EXTENSIONS[arrayLength(METALINK_EXTENSIONS)]);
-  return l;
-}
+namespace {
+const char* BT_EXTENSIONS[] = {".torrent", nullptr};
+} // namespace
 
-const std::vector<std::string>&
-DownloadHandlerConstants::getMetalinkContentTypes()
-{
-  static const std::vector<std::string> l
-    (&METALINK_CONTENT_TYPES[0],
-     &METALINK_CONTENT_TYPES[arrayLength(METALINK_CONTENT_TYPES)]);
-  return l;
-}
+const char** getBtExtensions() { return BT_EXTENSIONS; }
 
-const std::vector<std::string>& DownloadHandlerConstants::getBtExtensions()
-{
-  static const std::vector<std::string> l
-    (&BT_EXTENSIONS[0],
-     &BT_EXTENSIONS[arrayLength(BT_EXTENSIONS)]);
-  return l;
-}
+namespace {
+const char* BT_CONTENT_TYPES[] = {"application/x-bittorrent", nullptr};
+} // namespace
 
-const std::vector<std::string>& DownloadHandlerConstants::getBtContentTypes()
-{
-  static const std::vector<std::string> l
-    (&BT_CONTENT_TYPES[0],
-     &BT_CONTENT_TYPES[arrayLength(BT_CONTENT_TYPES)]);
-  return l;
-}
+const char** getBtContentTypes() { return BT_CONTENT_TYPES; }
 
 } // namespace aria2
