@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_HTTP_REQUEST_COMMAND_H_
-#define _D_HTTP_REQUEST_COMMAND_H_
+#ifndef D_HTTP_REQUEST_COMMAND_H
+#define D_HTTP_REQUEST_COMMAND_H
 
 #include "AbstractCommand.h"
 
@@ -42,38 +42,32 @@ namespace aria2 {
 class HttpConnection;
 class SocketCore;
 
-class HttpRequestCommand:public AbstractCommand {
+// HttpRequestCommand sends HTTP request header to remote server.
+// Because network I/O is non-blocking, execute() returns false if all
+// headers have not been sent. Subsequent calls of execute() send
+// remaining header and when all headers are completely sent,
+// execute() creates next Command object, HttpResponseCommand, and
+// returns true.
+class HttpRequestCommand : public AbstractCommand {
 private:
-  SharedHandle<Request> proxyRequest_;
+  std::shared_ptr<Request> proxyRequest_;
 
-  SharedHandle<HttpConnection> httpConnection_;
+  std::shared_ptr<HttpConnection> httpConnection_;
 
-  std::string connectedHostname_;
-  std::string connectedAddr_;
-  uint16_t connectedPort_;
 protected:
-  virtual bool executeInternal();
+  virtual bool executeInternal() CXX11_OVERRIDE;
+
 public:
-  HttpRequestCommand(cuid_t cuid,
-                     const SharedHandle<Request>& req,
-                     const SharedHandle<FileEntry>& fileEntry,
+  HttpRequestCommand(cuid_t cuid, const std::shared_ptr<Request>& req,
+                     const std::shared_ptr<FileEntry>& fileEntry,
                      RequestGroup* requestGroup,
-                     const SharedHandle<HttpConnection>& httpConnection,
-                     DownloadEngine* e,
-                     const SharedHandle<SocketCore>& s);
+                     const std::shared_ptr<HttpConnection>& httpConnection,
+                     DownloadEngine* e, const std::shared_ptr<SocketCore>& s);
   virtual ~HttpRequestCommand();
 
-  void setProxyRequest(const SharedHandle<Request>& proxyRequest);
-
-  void setConnectedAddr
-  (const std::string& hostname, const std::string& addr, uint16_t port)
-  {
-    connectedHostname_ = hostname;
-    connectedAddr_ = addr;
-    connectedPort_ = port;
-  }
+  void setProxyRequest(const std::shared_ptr<Request>& proxyRequest);
 };
 
 } // namespace aria2
 
-#endif // _D_HTTP_REQUEST_COMMAND_H_
+#endif // D_HTTP_REQUEST_COMMAND_H
