@@ -32,36 +32,27 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_HTTP_SERVER_RESPONSE_COMMAND_H_
-#define _D_HTTP_SERVER_RESPONSE_COMMAND_H_
+#ifndef D_HTTP_SERVER_RESPONSE_COMMAND_H
+#define D_HTTP_SERVER_RESPONSE_COMMAND_H
 
-#include "Command.h"
-#include "SharedHandle.h"
-#include "TimerA2.h"
+#include "AbstractHttpServerResponseCommand.h"
 
 namespace aria2 {
 
-class DownloadEngine;
-class SocketCore;
-class HttpServer;
+class HttpServerResponseCommand : public AbstractHttpServerResponseCommand {
+protected:
+  virtual void afterSend(const std::shared_ptr<HttpServer>& httpServer,
+                         DownloadEngine* e) CXX11_OVERRIDE;
 
-class HttpServerResponseCommand : public Command {
-private:
-  DownloadEngine* e_;
-  SharedHandle<SocketCore> socket_;
-  SharedHandle<HttpServer> httpServer_;
-  Timer timeoutTimer_;
 public:
   HttpServerResponseCommand(cuid_t cuid,
-                            const SharedHandle<HttpServer>& httpServer,
+                            const std::shared_ptr<HttpServer>& httpServer,
                             DownloadEngine* e,
-                            const SharedHandle<SocketCore>& socket);
+                            const std::shared_ptr<SocketCore>& socket);
 
   virtual ~HttpServerResponseCommand();
-  
-  virtual bool execute();
 };
 
-} // namespace aria2 
+} // namespace aria2
 
-#endif // _D_HTTP_SERVER_RESPONSE_COMMAND_H_
+#endif // D_HTTP_SERVER_RESPONSE_COMMAND_H
