@@ -38,20 +38,19 @@
 #include "EventPoll.h"
 
 #ifdef HAVE_PORT_H
-# include <port.h>
+#include <port.h>
 #endif // HAVE_PORT_H
 
-#include <deque>
+#include <set>
 
 #include "Event.h"
+#include "a2functional.h"
 #ifdef ENABLE_ASYNC_DNS
-# include "AsyncNameResolver.h"
+#include "AsyncNameResolver.h"
 #endif // ENABLE_ASYNC_DNS
 
 namespace aria2 {
 
-class Logger;
-
 class PortEventPoll : public EventPoll {
 private:
   class KSocketEntry;
@@ -67,8 +66,7 @@ private:
     KSocketEntry* socketEntry;
   };
 
-  class KSocketEntry:
-    public SocketEntry<KCommandEvent, KADNSEvent> {
+  class KSocketEntry : public SocketEntry<KCommandEvent, KADNSEvent> {
   public:
     KSocketEntry(sock_t socket);
 
@@ -78,9 +76,15 @@ private:
   friend int accumulateEvent(int events, const KEvent& event);
 
 private:
-  std::deque<SharedHandle<KSocketEntry> > socketEntries_;
+  typedef std::set<std::shared_ptr<KSocketEntry>,
+                   DerefLess<std::shared_ptr<KSocketEntry>>>
+      KSocketEntrySet;
+  KSocketEntrySet socketEntries_;
 #ifdef ENABLE_ASYNC_DNS
-  std::deque<SharedHandle<KAsyncNameResolverEntry> > nameResolverEntries_;
+  typedef std::set<std::shared_ptr<KAsyncNameResolverEntry>,
+                   DerefLess<std::shared_ptr<KAsyncNameResolverEntry>>>
+      KAsyncNameResolverEntrySet;
+  KAsyncNameResolverEntrySet nameResolverEntries_;
 #endif // ENABLE_ASYNC_DNS
 
   int port_;
@@ -91,17 +95,15 @@ private:
 
   static const size_t PORT_EVENTS_SIZE = 1024;
 
-  Logger* logger_;
-
   bool addEvents(sock_t socket, const KEvent& event);
 
   bool deleteEvents(sock_t socket, const KEvent& event);
 
   bool addEvents(sock_t socket, Command* command, int events,
-                 const SharedHandle<AsyncNameResolver>& rs);
+                 const std::shared_ptr<AsyncNameResolver>& rs);
 
   bool deleteEvents(sock_t socket, Command* command,
-                    const SharedHandle<AsyncNameResolver>& rs);
+                    const std::shared_ptr<AsyncNameResolver>& rs);
 
 public:
   PortEventPoll();
@@ -110,19 +112,21 @@ public:
 
   virtual ~PortEventPoll();
 
-  virtual void poll(const struct timeval& tv);
+  virtual void poll(const struct timeval& tv) CXX11_OVERRIDE;
 
-  virtual bool addEvents(sock_t socket,
-                         Command* command, EventPoll::EventType events);
+  virtual bool addEvents(sock_t socket, Command* command,
+                         EventPoll::EventType events) CXX11_OVERRIDE;
 
-  virtual bool deleteEvents(sock_t socket,
-                            Command* command, EventPoll::EventType events);
+  virtual bool deleteEvents(sock_t socket, Command* command,
+                            EventPoll::EventType events) CXX11_OVERRIDE;
 #ifdef ENABLE_ASYNC_DNS
 
-  virtual bool addNameResolver(const SharedHandle<AsyncNameResolver>& resolver,
-                               Command* command);
-  virtual bool deleteNameResolver
-  (const SharedHandle<AsyncNameResolver>& resolver, Command* command);
+  virtual bool
+  addNameResolver(const std::shared_ptr<AsyncNameResolver>& resolver,
+                  Command* command) CXX11_OVERRIDE;
+  virtual bool
+  deleteNameResolver(const std::shared_ptr<AsyncNameResolver>& resolver,
+                     Command* command) CXX11_OVERRIDE;
 #endif // ENABLE_ASYNC_DNS
 
   static const int IEV_READ = POLLIN;
