@@ -32,40 +32,47 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_TIME_BASED_COMMAND_H_
-#define _D_TIME_BASED_COMMAND_H_
+#ifndef D_TIME_BASED_COMMAND_H
+#define D_TIME_BASED_COMMAND_H
 
 #include "Command.h"
-#include "TimeA2.h"
+#include "TimerA2.h"
 
 namespace aria2 {
 
 class DownloadEngine;
 
-class TimeBasedCommand : public Command
-{
-protected:
-  DownloadEngine* _e;
+class TimeBasedCommand : public Command {
+private:
+  DownloadEngine* e_;
+
+  Timer checkPoint_;
+
+  std::chrono::seconds interval_;
 
   /**
-   * setting _exit to true if this command's job has finished and you want to
+   * setting exit_ to true if this command's job has finished and you want to
    * delete this command.
-   * The _exit variable is evaluated  after preProcess(), process(),
+   * The exit_ variable is evaluated  after preProcess(), process(),
    * postProcess(), and terminate processing immediately and excute() returns
    * true.
    */
-  bool _exit;
+  bool exit_;
 
-  time_t _interval; // unit: sec
+  bool routineCommand_;
+
+protected:
+  DownloadEngine* getDownloadEngine() const { return e_; }
+
+  void enableExit() { exit_ = true; }
+
+  const std::chrono::seconds& getInterval() const { return interval_; }
 
-  bool _routineCommand;
-private:
-  Time _checkPoint;
 public:
   /**
    * preProcess() is called each time when excute() is called.
    */
-  virtual void preProcess() {};
+  virtual void preProcess(){};
 
   /**
    * process() is called only when excute() is called and specified time has
@@ -76,17 +83,17 @@ public:
   /**
    * postProcess() is called each time when excute() is called.
    */
-  virtual void postProcess() {};
+  virtual void postProcess(){};
 
 public:
-  TimeBasedCommand(int32_t cuid, DownloadEngine* e, time_t interval,
-                   bool routineCommand = false);
+  TimeBasedCommand(cuid_t cuid, DownloadEngine* e,
+                   std::chrono::seconds interval, bool routineCommand = false);
 
   virtual ~TimeBasedCommand();
 
-  virtual bool execute();
+  virtual bool execute() CXX11_OVERRIDE;
 };
 
 } // namespace aria2
 
-#endif // _D_TIME_BASED_COMMAND_H_
+#endif // D_TIME_BASED_COMMAND_H
