@@ -95,7 +95,7 @@ typedef uint64_t A2Gid;
  *
  * The type of Key/Value pairs.
  */
-typedef std::vector<std::pair<std::string, std::string> > KeyVals;
+typedef std::vector<std::pair<std::string, std::string>> KeyVals;
 
 /**
  * @enum
@@ -120,7 +120,7 @@ enum DownloadEvent {
    */
   EVENT_ON_DOWNLOAD_COMPLETE,
   /**
-   * Indicating download has stopped becauseof the error.
+   * Indicating download has stopped because of the error.
    */
   EVENT_ON_DOWNLOAD_ERROR,
   /**
@@ -143,7 +143,7 @@ enum DownloadEvent {
  * of this callback should return 0 for compatibility.
  */
 typedef int (*DownloadEventCallback)(Session* session, DownloadEvent event,
-                                     const A2Gid& gid, void* userData);
+                                     A2Gid gid, void* userData);
 
 /**
  * @struct
@@ -257,7 +257,7 @@ int run(Session* session, RUN_MODE mode);
  *
  * Returns textual representation of the |gid|.
  */
-std::string gidToHex(const A2Gid& gid);
+std::string gidToHex(A2Gid gid);
 
 /**
  * @function
@@ -271,7 +271,7 @@ A2Gid hexToGid(const std::string& hex);
  *
  * Returns true if the |gid| is invalid.
  */
-bool isNull(const A2Gid& gid);
+bool isNull(A2Gid gid);
 
 /**
  * @function
@@ -291,11 +291,8 @@ bool isNull(const A2Gid& gid);
  * the queue, it is appended at the end of the queue.  This function
  * returns 0 if it succeeds, or negative error code.
  */
-int addUri(Session* session,
-           A2Gid* gid,
-           const std::vector<std::string>& uris,
-           const KeyVals& options,
-           int position = -1);
+int addUri(Session* session, A2Gid* gid, const std::vector<std::string>& uris,
+           const KeyVals& options, int position = -1);
 
 /**
  * @function
@@ -311,10 +308,8 @@ int addUri(Session* session,
  * the queue, it is appended at the end of the queue. This function
  * returns 0 if it succeeds, or negative error code.
  */
-int addMetalink(Session* session,
-                std::vector<A2Gid>* gids,
-                const std::string& metalinkFile,
-                const KeyVals& options,
+int addMetalink(Session* session, std::vector<A2Gid>* gids,
+                const std::string& metalinkFile, const KeyVals& options,
                 int position = -1);
 
 /**
@@ -339,12 +334,9 @@ int addMetalink(Session* session,
  * This function returns 0 if it succeeds, or negative error code.
  *
  */
-int addTorrent(Session* session,
-               A2Gid* gid,
-               const std::string& torrentFile,
+int addTorrent(Session* session, A2Gid* gid, const std::string& torrentFile,
                const std::vector<std::string>& webSeedUris,
-               const KeyVals& options,
-               int position = -1);
+               const KeyVals& options, int position = -1);
 
 /**
  * @function
@@ -352,11 +344,8 @@ int addTorrent(Session* session,
  * Same as :func:`addTorrent()` with an empty vector as the
  * |webSeedUris|.
  */
-int addTorrent(Session* session,
-               A2Gid* gid,
-               const std::string& torrentFile,
-               const KeyVals& options,
-               int position = -1);
+int addTorrent(Session* session, A2Gid* gid, const std::string& torrentFile,
+               const KeyVals& options, int position = -1);
 
 /**
  * @function
@@ -375,7 +364,7 @@ std::vector<A2Gid> getActiveDownload(Session* session);
  * takes time such as contacting BitTorrent tracker. This function
  * returns 0 if it succeeds, or negative error code.
  */
-int removeDownload(Session* session, const A2Gid& gid, bool force = false);
+int removeDownload(Session* session, A2Gid gid, bool force = false);
 
 /**
  * @function
@@ -395,7 +384,7 @@ int removeDownload(Session* session, const A2Gid& gid, bool force = false);
  * :member:`SessionConfig::keepRunning` to true. Otherwise, the
  * behavior is undefined.
  */
-int pauseDownload(Session* session, const A2Gid& gid, bool force = false);
+int pauseDownload(Session* session, A2Gid gid, bool force = false);
 
 /**
  * @function
@@ -405,7 +394,7 @@ int pauseDownload(Session* session, const A2Gid& gid, bool force = false);
  * makes the download eligible to restart. This function returns 0 if
  * it succeeds, or negative error code.
  */
-int unpauseDownload(Session* session, const A2Gid& gid);
+int unpauseDownload(Session* session, A2Gid gid);
 
 /**
  * @function
@@ -437,7 +426,7 @@ int unpauseDownload(Session* session, const A2Gid& gid);
  *
  * This function returns 0 if it succeeds, or negative error code.
  */
-int changeOption(Session* session, const A2Gid& gid, const KeyVals& options);
+int changeOption(Session* session, A2Gid gid, const KeyVals& options);
 
 /**
  * @function
@@ -571,8 +560,7 @@ enum OffsetMode {
  * This function returns the final destination position of this
  * download, or negative error code.
  */
-int changePosition(Session* session, const A2Gid& gid, int pos,
-                   OffsetMode how);
+int changePosition(Session* session, A2Gid gid, int pos, OffsetMode how);
 
 /**
  * @function
@@ -692,7 +680,7 @@ struct BtMetaInfoData {
    * ``announce`` and no ``announce-list``, ``announce`` is converted
    * to ``announce-list`` format.
    */
-  std::vector<std::vector<std::string> > announceList;
+  std::vector<std::vector<std::string>> announceList;
   /**
    * ``comment`` for the torrent. ``comment.utf-8`` is used if
    * available.
@@ -753,7 +741,7 @@ enum DownloadStatus {
  */
 class DownloadHandle {
 public:
-  virtual ~DownloadHandle() {}
+  virtual ~DownloadHandle() = default;
   /**
    * Returns status of this download.
    */
@@ -819,6 +807,12 @@ public:
    * downloads, this function returns empty array.
    */
   virtual const std::vector<A2Gid>& getFollowedBy() = 0;
+  /**
+   * Returns the GID of the download which generated this download.
+   * This is a reverse link of
+   * :func:`DownloadHandle::getFollowedBy()`.
+   */
+  virtual A2Gid getFollowing() = 0;
   /**
    * Returns the GID of a parent download. Some downloads are a part
    * of another download. For example, if a file in Metalink has
@@ -867,10 +861,6 @@ public:
    * not return options which have no default value and have not been
    * set by :func:`sessionNew()`, configuration files or API
    * functions.
-   *
-   * Calling this function for the download which is not in
-   * :c:macro:`DOWNLOAD_ACTIVE`, :c:macro:`DOWNLOAD_PAUSED` or
-   * :c:macro:`DOWNLOAD_WAITING` will return empty array.
    */
   virtual KeyVals getOptions() = 0;
 };
@@ -888,7 +878,7 @@ public:
  * responsibility of the caller to call :func:`deleteDownloadHandle()`
  * to delete handle object.
  */
-DownloadHandle* getDownloadHandle(Session* session, const A2Gid& gid);
+DownloadHandle* getDownloadHandle(Session* session, A2Gid gid);
 
 /**
  * @function
